	program simcr
!++
!  Title:
!	SIMCR - Simulate input from a card reader
!
!  Facility:
!  	System utility to simulate input from a card reader
!
!  Abstract:
!  	This utility is used to submit a file to the system batch queues
!  	as if the job was read in through a card eater...I mean card
!  	reader.  The file must begin with $JOB and $PASSWORD commands
!  	which allow specifying the job attributes (including of course
!  	the username).  The file to be submitted is expected to be 
!  	located on SYS$INPUT.
!
!  	Some special handling has been incorporated to support
!  	submitting files returned via a HASP punch stream.
!  	Any lines beginning with a /* are assumed to be delimiters
!  	and are discarded.  The SYS$INPUT file is deleted following
!  	transmission.
!  
!  	The mechanism for submitting the job is as follows:  A 
!  	mailbox is created to be the actual input device.  A
!  	message is sent to JOB_CONTROL on his mailbox indicating
!  	"unsolicited card-reader input" and specifying the input
!	mailbox as the card reader.  JOB_CONTROL fires up a subprocess
!  	running the input symbiont (SYS$SYSTEM:INPSMB) which does the
!  	rest.  Note that currently INPSMB works without any
!	modifications because it doesn't do anything which is illegal
!  	on a mailbox.  However, this could change in the future
!  	requiring a patch to INPSMB.
!
!       In order for this utility to send a message to JOB_CONTROL
!  	it must obtain write access to the JOB_CONTROL mailbox.  In
!  	order to do this, SIMCR turns on BYPASS priv when writing to
!  	the mailbox.  Therefore, it must be installed with SETPRV or
!  	used by a user with this privilege.
!  
!  Environment:
!	Native Mode. Must be installed with or run by a user with 
!  	SETPRV priv.
!
!  Author: 
!	Gary L. Grebus, Creation date: 21-Dec-1982
!	Battelle Columbus Labs
!
!  Modified by:
!
!  	1.01 - Gary L. Grebus, 9-Feb-1983
!  		Don't delete first card in file.  It is suppressed by
!  		option on CDC punch queue.
!
! 	1.02 - Gary L. Grebus, 18-May-1983 13:55:39
! 	        Discard only lines which are exactly /*EOx.
!--


!
!  Constant definitions
!
	include '($iodef)/nolist'      !  QIO symbols
	include '($ssdef)/nolist'      !  System service status codes

	parameter input_unit = 1       !  FORTRAN unit for input file
	parameter input_recsize = 80   !  Max size of input record
!
!  Local storage
!
	integer*2 mbx_chan	       !  Channel on output mailbox
	integer*2 mbx_iosb(4)	       !  IOSB for output mailbox I/O
	integer*2 mbx_devunit	       !  Device unit nr of output mailbox

	character*(input_recsize) buffer      !  Buffer for input record
	integer*2 buffer_length	       !  Length of record in buffer
	integer*4 status	       !  Scratch for return status

!
! Function type definitions
!
	integer*4 get_mbx	       !  Function to obtain output
				       !  mailbox
	integer*4 send_jbc_msg	       !  Function to send "card
				       !  reader" message to job
				       !  control
	integer*4 sys$qiow
	integer*4 sys$dassgn

!  Obtain the output mailbox
	status = get_mbx(mbx_chan, mbx_devunit)
	if (.not. status) then
	    call lib$stop(%val(status))
	endif

!  Open the input file
	open (name='SYS$INPUT', unit=input_unit, type='OLD')

!  Tell JOB_CONTROL that the "card reader" is ready with data
	status = send_jbc_msg (mbx_devunit)
	if (.not. status) then
	    call lib$stop(%val(status))
	endif


!  Loop processing records
	do while(.true.)

	    read (unit=input_unit,fmt=100,end=200) buffer_length, buffer
100	    format(q, a)

	    if (buffer(1:4) .ne. '/*EO'.or.
	1        buffer_length .ne. 5) then !  Discard /*EOx lines

		status = sys$qiow(, %val(mbx_chan), 
	1		%val(io$_writevblk),
	1		mbx_iosb, , , %ref(buffer),
	1		%val(buffer_length),,,,)
		if (status) then       !  If QIO succeeded
		    status = mbx_iosb(1) !  Get I/O status
		endif

		if (.not. status) then
		    call sys$qiow(, %val(mbx_chan),
	1		%val(io$_writeof),,,,,,,,,) !  Drop EOF on
				       !  mailbox
		    call lib$stop(%val(status))
		endif

	    endif

	end do

!  Here on EOF on input file.  The DELETE disposition is also peculiar
!  to HASP application.

200	close(unit=input_unit,disp='DELETE')

!  Drop an EOF on the mailbox
	status = sys$qiow(, %val(mbx_chan), 
	1	%val(io$_writeof),
	1	mbx_iosb, , , , , , , , )
	if (status) then       !  If QIO succeeded
	    status = mbx_iosb(1) !  Get I/O status
	endif

	if (.not. status) then
	    call lib$stop(%val(status))
	endif

!  All done.  Deassign mailbox and terminate with success.
	call sys$dassgn(%val(mbx_chan))
	call sys$exit(%val(ss$_normal))

	end

	integer function get_mbx (chan, devunit)

!++
!  Functional Description:
!	This routine is used to obtain a temporary mailbox and return
!  	its device unit number and a channel on the device.
!
!  Calling Sequence:
!  	i = get_mbx(chan, devunit)
!
!  Input Parameters:  NONE
!	
!  Output Parameters:
!  	chan - channel number for the channel assigned (by reference)
!  	devunit - device unit of the mailbox created (by reference)
!
!  Implicit Inputs: NONE
!
!  Implicit Outputs: NONE
!
!  Procedures called:
!  	sys$crembx, sys$getdvi
!
!  Completion Status:
!  	Returns ss$_normal if successful.
!  	Returns any error status from system services calles.
!
!  Side Effects:  NONE
!
!--

!
!  Constant definitions
!
	include '(dvidef)/nolist'      !  $getdvi symbols
	parameter input_recsize = 80   !  Max size of input record
	parameter mbx_protection = 'ff00'x !  Protection on mailbox created
				       !  (S:RW,O:RW,G,W)

!
!  Formal parameters
!  
	integer*2 chan
	integer*2 devunit

!  
!  Local storage
!
	integer*2 dvi_itemlist(8)      !  $getdvi request list
	integer*4 dvi_bufaddr(3)
	equivalence (dvi_itemlist(3), dvi_bufaddr(1))

	integer*4 status	       !  Scratch for return status

!
! Function type definitions
!
	integer*4 sys$crembx
	integer*4 sys$getdvi

!  Get the mailbox channel

	status = sys$crembx(%val(0), chan, %val(input_recsize), ,
	1	%val(mbx_protection), , )

	if (status) then

!  Return the device unit number
	    dvi_itemlist(1) = 4	       !  Build the request list
	    dvi_itemlist(2) = dvi$_unit
	    dvi_bufaddr(1) = %loc(i)
	    dvi_bufaddr(2) = 0
	    dvi_bufaddr(3) = 0	       !  List terminator

	    status = sys$getdvi(, %val(chan), , dvi_itemlist,,,,)
	    devunit = i

	endif

	get_mbx = status

	return
	end
	integer function send_jbc_msg (devunit)

!++
!  Functional Description:
!  	This routine is used to send a "unsolicited input"
!  	message on the JOB_CONTROL mailbox (MBA1:).
!  	The device which is identified in the message is passed as
!  	a parameter.  To insure we can send the message, we set
!  	BYPASS priv during the mailbox write.
!
!  Calling Sequence:
!  	i = send_jbc_msg(devunit)
!
!  Input Parameters:  
!  	devunit - Name of device to pass to JBC (by descriptor)
!	
!  Output Parameters:  NONE
!
!  Implicit Inputs: NONE
!
!  Implicit Outputs: NONE
!
!  Procedures called:
!  	sys$assign, sys$qiow, sys$setprv, sys$dassgn
!
!  Completion Status:
!  	Returns ss$_normal if successful.
!  	Returns any error status from system services calles.
!
!  Side Effects: 
!  	Causes JOB_CONTROL to create a detached process running
!  	the input symbiont and reading data from the mailbox device
!  	whose unit is passed in devunit.
!
!--

!
!  Constant definitions
!
	include '($iodef)/nolist'      !  QIO symbols
	include '($msgdef)/nolist'     !  JOB CONTROL message types
	include '($prvdef)/nolist'     !  Privilege bit definitions

!
!  Formal parameters
!  
	integer*2 devunit

!  
!  Local storage
!
	integer*2 mba1_chan	       !  Channel obtained on mailbox
	integer*2 mba1_iosb(4)	       !  IOSB for mailbox I/O

	integer*2 mba1_msg(4)	       !  Message to be sent to JOB
				       !  CONTROL
	byte dev_size(2)
	character*3 dev_ctrl
	equivalence (dev_size(1), mba1_msg(3))
	equivalence (dev_ctrl, dev_size(2))

	integer*4 priv_mask(2)	       !  Mask for enabling BYPASS
	integer*4 prev_priv(2)	       !  Space for storing old privs

	integer*4 status	       !  Scratch for return status

!
! Function type definitions
!
	integer*4 sys$assign
	integer*4 sys$qiow
	integer*4 sys$dassgn
	integer*4 sys$setprv

!  Build the message and the priv mask
	dev_size(1) = 3		       !  Length of dev name
	dev_ctrl = 'MBA'	       !  Device name
	mba1_msg(1) = msg$_crunsolic   !  Message type
	mba1_msg(2) = devunit

	priv_mask(1) =  ibset(0, prv$v_bypass) !  Priv mask
	priv_mask(2) = 0

!  Get a channel on the mailbox
	status = sys$assign('_MBA1:', mba1_chan, , )

	if (status) then

!  Switch privs and send the message
	    status = sys$setprv(%val(1), priv_mask, %val(0), prev_priv)
	    if (status) then
		status = sys$qiow(, %val(mba1_chan),
	1		%val(io$_writevblk),
	1		mba1_iosb, , , mba1_msg,
	1		%val(8),,,,)   !  Send the message
		if (status) then
		    status = mba1_iosb(1) !  If QIO worked, get IO
				       !  status
		endif

		call sys$setprv(%val(1), prev_priv, %val(0), )
	    endif

	    call sys$dassgn(%val(mba1_chan))	!  Free mailbox channel

	endif

	send_jbc_msg = status	       !  Return last status

	return
	end
